package io.horizontalsystems.bankwallet.modules.ratelist

import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.core.view.isVisible
import androidx.recyclerview.widget.DiffUtil
import androidx.recyclerview.widget.ListAdapter
import androidx.recyclerview.widget.RecyclerView
import io.horizontalsystems.bankwallet.R
import io.horizontalsystems.bankwallet.core.setCoinImage
import kotlinx.android.extensions.LayoutContainer
import kotlinx.android.synthetic.main.view_holder_coin_rate.*

class CoinRatesAdapter(private val listener: Listener) : ListAdapter<CoinItem, ViewHolderCoin>(coinRateDiff) {

    interface Listener {
        fun onCoinClicked(coinItem: CoinItem)
    }

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): ViewHolderCoin {
        return ViewHolderCoin(LayoutInflater.from(parent.context).inflate(R.layout.view_holder_coin_rate, parent, false), listener)
    }

    override fun onBindViewHolder(holder: ViewHolderCoin, position: Int) {
        holder.bind(getItem(position))
    }

    companion object {
        private val coinRateDiff = object: DiffUtil.ItemCallback<CoinItem>() {
            override fun areItemsTheSame(oldItem: CoinItem, newItem: CoinItem): Boolean {
                return oldItem == newItem
            }

            override fun areContentsTheSame(oldItem: CoinItem, newItem: CoinItem): Boolean {
                return oldItem == newItem
            }
        }
    }
}

class ViewHolderCoin(override val containerView: View, listener: CoinRatesAdapter.Listener) : RecyclerView.ViewHolder(containerView), LayoutContainer {
    private var coinItem: CoinItem? = null

    init {
        containerView.setOnClickListener {
            coinItem?.let {
                listener.onCoinClicked(it)
            }
        }
    }

    fun bind(coinItem: CoinItem) {
        this.coinItem = coinItem

        if (coinItem.coin != null) {
            coinIcon.isVisible = true
            coinIcon.setCoinImage(coinItem.coin.code, coinItem.coin.type)
        } else {
            coinIcon.isVisible = false
        }

        if (coinItem.rank != null) {
            rank.isVisible = true
            rank.text = coinItem.rank.toString()
        } else {
            rank.isVisible = false
        }

        titleText.text = coinItem.coinName
        subtitleText.text = coinItem.coinCode

        txValueInFiat.isActivated = !coinItem.rateDimmed //change color via state: activated/not activated
        txValueInFiat.text = coinItem.rate ?: containerView.context.getString(R.string.NotAvailable)

        txDiff.isVisible = coinItem.diff != null
        txDiffNa.isVisible = coinItem.diff == null

        if (coinItem.diff != null) {
            txDiff.diff = coinItem.diff
        }
    }
}
